package javax.swing.plaf.basic;

import sun.swing.UIAction;
import java.awt.event.KeyEvent;
import java.awt.ComponentOrientation;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.awt.Color;
import javax.swing.JTextField;
import java.util.Locale;
import java.awt.event.ActionListener;
import javax.swing.JRootPane;
import java.awt.event.HierarchyEvent;
import java.awt.event.HierarchyListener;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.KeyStroke;
import javax.swing.JScrollPane;
import java.awt.Font;
import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.Icon;
import java.awt.Insets;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.BorderLayout;
import javax.swing.border.Border;
import javax.swing.JPanel;
import sun.swing.DefaultLookup;
import javax.swing.InputMap;
import javax.swing.SwingUtilities;
import javax.swing.BoxLayout;
import java.awt.LayoutManager;
import java.awt.Container;
import javax.swing.UIManager;
import javax.swing.LookAndFeel;
import javax.swing.plaf.ComponentUI;
import java.beans.PropertyChangeListener;
import java.awt.Component;
import javax.swing.JComponent;
import java.awt.Dimension;
import javax.swing.JOptionPane;
import javax.swing.plaf.OptionPaneUI;

public class BasicOptionPaneUI extends OptionPaneUI
{
    public static final int MinimumWidth = 262;
    public static final int MinimumHeight = 90;
    private static String newline;
    protected JOptionPane optionPane;
    protected Dimension minimumSize;
    protected JComponent inputComponent;
    protected Component initialFocusComponent;
    protected boolean hasCustomComponents;
    protected PropertyChangeListener propertyChangeListener;
    private BasicOptionPaneUI.Handler handler;
    
    public BasicOptionPaneUI() {
    }
    
    static void loadActionMap(LazyActionMap lazyActionMap) {
        lazyActionMap.put(new BasicOptionPaneUI.Actions("close"));
        BasicLookAndFeel.installAudioActionMap(lazyActionMap);
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new BasicOptionPaneUI();
    }
    
    @Override
    public void installUI(JComponent component) {
        this.optionPane = (JOptionPane)component;
        this.installDefaults();
        this.optionPane.setLayout(this.createLayoutManager());
        this.installComponents();
        this.installListeners();
        this.installKeyboardActions();
    }
    
    @Override
    public void uninstallUI(JComponent component) {
        this.uninstallComponents();
        this.optionPane.setLayout(null);
        this.uninstallKeyboardActions();
        this.uninstallListeners();
        this.uninstallDefaults();
        this.optionPane = null;
    }
    
    protected void installDefaults() {
        LookAndFeel.installColorsAndFont(this.optionPane, "OptionPane.background", "OptionPane.foreground", "OptionPane.font");
        LookAndFeel.installBorder(this.optionPane, "OptionPane.border");
        this.minimumSize = UIManager.getDimension("OptionPane.minimumSize");
        LookAndFeel.installProperty(this.optionPane, "opaque", Boolean.TRUE);
    }
    
    protected void uninstallDefaults() {
        LookAndFeel.uninstallBorder(this.optionPane);
    }
    
    protected void installComponents() {
        this.optionPane.add(this.createMessageArea());
        Container separator = this.createSeparator();
        if (separator != null) {
            this.optionPane.add(separator);
        }
        this.optionPane.add(this.createButtonArea());
        this.optionPane.applyComponentOrientation(this.optionPane.getComponentOrientation());
    }
    
    protected void uninstallComponents() {
        this.hasCustomComponents = false;
        this.inputComponent = null;
        this.initialFocusComponent = null;
        this.optionPane.removeAll();
    }
    
    protected LayoutManager createLayoutManager() {
        return new BoxLayout(this.optionPane, 1);
    }
    
    protected void installListeners() {
        PropertyChangeListener propertyChangeListener = this.createPropertyChangeListener();
        this.propertyChangeListener = propertyChangeListener;
        if (propertyChangeListener != null) {
            this.optionPane.addPropertyChangeListener(this.propertyChangeListener);
        }
    }
    
    protected void uninstallListeners() {
        if (this.propertyChangeListener != null) {
            this.optionPane.removePropertyChangeListener(this.propertyChangeListener);
            this.propertyChangeListener = null;
        }
        this.handler = null;
    }
    
    protected PropertyChangeListener createPropertyChangeListener() {
        return this.getHandler();
    }
    
    private BasicOptionPaneUI.Handler getHandler() {
        if (this.handler == null) {
            this.handler = new BasicOptionPaneUI.Handler();
        }
        return this.handler;
    }
    
    protected void installKeyboardActions() {
        SwingUtilities.replaceUIInputMap(this.optionPane, 2, this.getInputMap(2));
        LazyActionMap.installLazyActionMap(this.optionPane, BasicOptionPaneUI.class, "OptionPane.actionMap");
    }
    
    protected void uninstallKeyboardActions() {
        SwingUtilities.replaceUIInputMap(this.optionPane, 2, null);
        SwingUtilities.replaceUIActionMap(this.optionPane, null);
    }
    
    InputMap getInputMap(int n) {
        if (n == 2) {
            Object[] keys = (Object[])(Object[])DefaultLookup.get(this.optionPane, this, "OptionPane.windowBindings");
            if (keys != null) {
                return LookAndFeel.makeComponentInputMap(this.optionPane, keys);
            }
        }
        return null;
    }
    
    public Dimension getMinimumOptionPaneSize() {
        if (this.minimumSize == null) {
            return new Dimension(262, 90);
        }
        return new Dimension(this.minimumSize.width, this.minimumSize.height);
    }
    
    @Override
    public Dimension getPreferredSize(JComponent component) {
        if (component == this.optionPane) {
            Dimension minimumOptionPaneSize = this.getMinimumOptionPaneSize();
            LayoutManager layout = component.getLayout();
            if (layout != null) {
                Dimension preferredLayoutSize = layout.preferredLayoutSize(component);
                if (minimumOptionPaneSize != null) {
                    return new Dimension(Math.max(preferredLayoutSize.width, minimumOptionPaneSize.width), Math.max(preferredLayoutSize.height, minimumOptionPaneSize.height));
                }
                return preferredLayoutSize;
            }
            else {
                return minimumOptionPaneSize;
            }
        }
        else {
            return null;
        }
    }
    
    protected Container createMessageArea() {
        JPanel panel = new JPanel();
        Border border = (Border)DefaultLookup.get(this.optionPane, this, "OptionPane.messageAreaBorder");
        if (border != null) {
            panel.setBorder(border);
        }
        panel.setLayout(new BorderLayout());
        JPanel comp = new JPanel(new GridBagLayout());
        JPanel comp2 = new JPanel(new BorderLayout());
        comp.setName("OptionPane.body");
        comp2.setName("OptionPane.realBody");
        if (this.getIcon() != null) {
            JPanel comp3 = new JPanel();
            comp3.setName("OptionPane.separator");
            comp3.setPreferredSize(new Dimension(15, 1));
            comp2.add(comp3, "Before");
        }
        comp2.add(comp, "Center");
        GridBagConstraints gridBagConstraints;
        Object o = gridBagConstraints = new GridBagConstraints();
        int n = 0;
        o.gridy = n;
        gridBagConstraints.gridx = n;
        o.gridwidth = 0;
        o.gridheight = 1;
        o.anchor = DefaultLookup.getInt(this.optionPane, this, "OptionPane.messageAnchor", 10);
        o.insets = new Insets(0, 0, 3, 0);
        this.addMessageComponents(comp, o, this.getMessage(), this.getMaxCharactersPerLineCount(), false);
        panel.add(comp2, "Center");
        this.addIcon(panel);
        return panel;
    }
    
    protected void addMessageComponents(Container container, GridBagConstraints constraints, Object o, int n, boolean b) {
        if (o == null) {
            return;
        }
        if ((o instanceof Component)) {
            if (o instanceof JScrollPane || o instanceof JPanel) {
                constraints.fill = 1;
                constraints.weighty = 1.0;
            }
            else {
                constraints.fill = 2;
            }
            constraints.weightx = 1.0;
            container.add((Component)o, constraints);
            constraints.weightx = 0.0;
            constraints.weighty = 0.0;
            constraints.fill = 0;
            ++constraints.gridy;
            if (!b) {
                this.hasCustomComponents = true;
            }
        }
        else if ((o instanceof Object[])) {
            Object[] array = (Object[])(Object[])o;
            for (int length3 = array.length, i = 0; i < length3; ++i) {
                this.addMessageComponents(container, constraints, array[i], n, false);
            }
        }
        else if ((o instanceof Icon)) {
            JLabel label = new JLabel((Icon)o, 0);
            this.configureMessageLabel(label);
            this.addMessageComponents(container, constraints, label, n, true);
        }
        else {
            String string = o.toString();
            int length = string.length();
            if (length <= 0) {
                return;
            }
            int length2 = 0;
            int endIndex;
            if ((endIndex = string.indexOf(BasicOptionPaneUI.newline)) >= 0) {
                length2 = BasicOptionPaneUI.newline.length();
            }
            else if ((endIndex = string.indexOf("\r\n")) >= 0) {
                length2 = 2;
            }
            else if ((endIndex = string.indexOf(10)) >= 0) {
                length2 = 1;
            }
            if (endIndex >= 0) {
                if (endIndex == 0) {
                    BasicOptionPaneUI.1 o3 = new JPanel() {
                        @Override
                        public Dimension getPreferredSize() {
                            Font font = this.getFont();
                            if (font != null) {
                                return new Dimension(1, font.getSize() + 2);
                            }
                            return new Dimension(0, 0);
                        }
                    };
                    o3.setName("OptionPane.break");
                    this.addMessageComponents(container, constraints, o3, n, true);
                }
                else {
                    this.addMessageComponents(container, constraints, string.substring(0, endIndex), n, false);
                }
                this.addMessageComponents(container, constraints, string.substring(endIndex + length2), n, false);
            }
            else if (length > n) {
                Box verticalBox = Box.createVerticalBox();
                verticalBox.setName("OptionPane.verticalBox");
                this.burstStringInto(verticalBox, string, n);
                this.addMessageComponents(container, constraints, verticalBox, n, true);
            }
            else {
                JLabel o2 = new JLabel(string, 10);
                o2.setName("OptionPane.label");
                this.configureMessageLabel(o2);
                this.addMessageComponents(container, constraints, o2, n, true);
            }
        }
        return;
    }
    
    protected Object getMessage() {
        this.inputComponent = null;
        if (this.optionPane != null) {
            if (this.optionPane.getWantsInput()) {
                Object message = this.optionPane.getMessage();
                Object[] selectionValues = this.optionPane.getSelectionValues();
                Object initialSelectionValue = this.optionPane.getInitialSelectionValue();
                BasicOptionPaneUI.MultiplexingTextField multiplexingTextField;
                if (selectionValues != null) {
                    if (selectionValues.length < 20) {
                        JComboBox<Object> comboBox = new JComboBox();
                        multiplexingTextField.setName("OptionPane.comboBox");
                        for (int i = 0; i < selectionValues.length; ++i) {
                            multiplexingTextField.addItem(selectionValues[i]);
                        }
                        if (initialSelectionValue != null) {
                            multiplexingTextField.setSelectedItem(initialSelectionValue);
                        }
                        this.inputComponent = multiplexingTextField;
                    }
                    else {
                        JList<Object> list = new JList(selectionValues);
                        new JScrollPane(list).setName("OptionPane.scrollPane");
                        list.setName("OptionPane.list");
                        list.setVisibleRowCount(10);
                        list.setSelectionMode(0);
                        if (initialSelectionValue != null) {
                            list.setSelectedValue(initialSelectionValue, true);
                        }
                        list.addMouseListener(this.getHandler());
                        this.inputComponent = list;
                    }
                }
                else {
                    multiplexingTextField = new BasicOptionPaneUI.MultiplexingTextField(20);
                    multiplexingTextField.setName("OptionPane.textField");
                    multiplexingTextField.setKeyStrokes(new KeyStroke[] { KeyStroke.getKeyStroke("ENTER") });
                    if (initialSelectionValue != null) {
                        String string = initialSelectionValue.toString();
                        multiplexingTextField.setText(string);
                        multiplexingTextField.setSelectionStart(0);
                        multiplexingTextField.setSelectionEnd(string.length());
                    }
                    multiplexingTextField.addActionListener(this.getHandler());
                    this.inputComponent = multiplexingTextField;
                }
                if (message == null) {
                    return new Object[] { multiplexingTextField };
                }
                return new Object[] { message, multiplexingTextField };
            }
            else {
                return this.optionPane.getMessage();
            }
        }
        else {
            return null;
        }
    }
    
    protected void addIcon(Container container) {
        Icon icon = this.getIcon();
        if (icon != null) {
            JLabel comp = new JLabel(icon);
            comp.setName("OptionPane.iconLabel");
            comp.setVerticalAlignment(1);
            container.add(comp, "Before");
        }
    }
    
    protected Icon getIcon() {
        Icon iconForType = (this.optionPane == null) ? null : this.optionPane.getIcon();
        if (iconForType == null && this.optionPane != null) {
            iconForType = this.getIconForType(this.optionPane.getMessageType());
            return iconForType;
        }
        return iconForType;
    }
    
    protected Icon getIconForType(int n) {
        if (n < 0 || n > 3) {
            return null;
        }
        String key = null;
        switch (n) {
            case 0: {
                key = "OptionPane.errorIcon";
                break;
            }
            case 1: {
                key = "OptionPane.informationIcon";
                break;
            }
            case 2: {
                key = "OptionPane.warningIcon";
                break;
            }
            case 3: {
                key = "OptionPane.questionIcon";
                break;
            }
        }
        if (key != null) {
            return (Icon)DefaultLookup.get(this.optionPane, this, key);
        }
        return null;
    }
    
    protected int getMaxCharactersPerLineCount() {
        return this.optionPane.getMaxCharactersPerLineCount();
    }
    
    protected void burstStringInto(Container container, String text, int n) {
        int length = text.length();
        if (length <= 0) {
            return;
        }
        if (length > n) {
            int endIndex = text.lastIndexOf(32, n);
            if (endIndex <= 0) {
                endIndex = text.indexOf(32, n);
            }
            if (endIndex > 0 && endIndex < length) {
                this.burstStringInto(container, text.substring(0, endIndex), n);
                this.burstStringInto(container, text.substring(endIndex + 1), n);
                return;
            }
        }
        JLabel comp = new JLabel(text, 2);
        comp.setName("OptionPane.label");
        this.configureMessageLabel(comp);
        container.add(comp);
        return;
    }
    
    protected Container createSeparator() {
        return null;
    }
    
    protected Container createButtonArea() {
        JPanel panel = new JPanel();
        Border border = (Border)DefaultLookup.get(this.optionPane, this, "OptionPane.buttonAreaBorder");
        panel.setName("OptionPane.buttonArea");
        if (border != null) {
            panel.setBorder(border);
        }
        panel.setLayout(new BasicOptionPaneUI.ButtonAreaLayout(DefaultLookup.getBoolean(this.optionPane, this, "OptionPane.sameSizeButtons", true), DefaultLookup.getInt(this.optionPane, this, "OptionPane.buttonPadding", 6), DefaultLookup.getInt(this.optionPane, this, "OptionPane.buttonOrientation", 0), DefaultLookup.getBoolean(this.optionPane, this, "OptionPane.isYesLast", false)));
        this.addButtonComponents(panel, this.getButtons(), this.getInitialValueIndex());
        return panel;
    }
    
    protected void addButtonComponents(Container container, Object[] array, int n) {
        if (array != null && array.length > 0) {
            int sizeButtonsToSameWidth = this.getSizeButtonsToSameWidth();
            boolean n2 = true;
            int length = array.length;
            JButton[] array2 = null;
            int max = 0;
            if (!(sizeButtonsToSameWidth == 0)) {
                array2 = new JButton[length];
            }
            for (int i = 0; i < length; ++i) {
                Object o = array[i];
                Component component;
                if ((o instanceof Component)) {
                    n2 = false;
                    component = (Component)o;
                    container.add(component);
                    this.hasCustomComponents = true;
                }
                else {
                    JButton button;
                    if ((o instanceof BasicOptionPaneUI.ButtonFactory)) {
                        button = ((BasicOptionPaneUI.ButtonFactory)o).createButton();
                    }
                    else if ((o instanceof Icon)) {
                        button = new JButton((Icon)o);
                    }
                    else {
                        button = new JButton(o.toString());
                    }
                    button.setName("OptionPane.button");
                    button.setMultiClickThreshhold((long)DefaultLookup.getInt(this.optionPane, this, "OptionPane.buttonClickThreshhold", 0));
                    this.configureButton(button);
                    container.add(button);
                    ActionListener buttonActionListener = this.createButtonActionListener(i);
                    if (buttonActionListener != null) {
                        button.addActionListener(buttonActionListener);
                    }
                    component = button;
                }
                if (sizeButtonsToSameWidth && n2 && component instanceof JButton) {
                    array2[i] = (JButton)component;
                    max = Math.max(max, component.getMinimumSize().width);
                }
                if (i == n) {
                    this.initialFocusComponent = component;
                    if ((this.initialFocusComponent instanceof JButton)) {
                        ((Component)this.initialFocusComponent).addHierarchyListener(new HierarchyListener() {
                            @Override
                            public void hierarchyChanged(HierarchyEvent hierarchyEvent) {
                                if ((hierarchyEvent.getChangeFlags() & 0x1L) != 0x0L) {
                                    JButton button = (JButton)hierarchyEvent.getComponent();
                                    JRootPane rootPane = SwingUtilities.getRootPane(button);
                                    if (rootPane != null) {
                                        rootPane.setDefaultButton(button);
                                    }
                                }
                            }
                        });
                    }
                }
            }
            ((BasicOptionPaneUI.ButtonAreaLayout)container.getLayout()).setSyncAllWidths(sizeButtonsToSameWidth && n2);
            if (DefaultLookup.getBoolean(this.optionPane, this, "OptionPane.setButtonMargin", true) && sizeButtonsToSameWidth && n2) {
                int n3 = (length <= 2) ? 8 : 4;
                for (int j = 0; j < length; ++j) {
                    array2[j].setMargin(new Insets(2, n3, 2, n3));
                }
            }
        }
    }
    
    protected ActionListener createButtonActionListener(int n) {
        return new BasicOptionPaneUI.ButtonActionListener(n);
    }
    
    protected Object[] getButtons() {
        if (this.optionPane != null) {
            Object[] options = this.optionPane.getOptions();
            if (options == null) {
                int optionType = this.optionPane.getOptionType();
                Locale locale = this.optionPane.getLocale();
                int int1 = DefaultLookup.getInt(this.optionPane, this, "OptionPane.buttonMinimumWidth", -1);
                BasicOptionPaneUI.ButtonFactory[] array;
                if (optionType == 0) {
                    array = new BasicOptionPaneUI.ButtonFactory[] { new BasicOptionPaneUI.ButtonFactory(UIManager.getString("OptionPane.yesButtonText", locale), this.getMnemonic("OptionPane.yesButtonMnemonic", locale), (Icon)DefaultLookup.get(this.optionPane, this, "OptionPane.yesIcon"), int1), new BasicOptionPaneUI.ButtonFactory(UIManager.getString("OptionPane.noButtonText", locale), this.getMnemonic("OptionPane.noButtonMnemonic", locale), (Icon)DefaultLookup.get(this.optionPane, this, "OptionPane.noIcon"), int1) };
                }
                else if (optionType == 1) {
                    array = new BasicOptionPaneUI.ButtonFactory[] { new BasicOptionPaneUI.ButtonFactory(UIManager.getString("OptionPane.yesButtonText", locale), this.getMnemonic("OptionPane.yesButtonMnemonic", locale), (Icon)DefaultLookup.get(this.optionPane, this, "OptionPane.yesIcon"), int1), new BasicOptionPaneUI.ButtonFactory(UIManager.getString("OptionPane.noButtonText", locale), this.getMnemonic("OptionPane.noButtonMnemonic", locale), (Icon)DefaultLookup.get(this.optionPane, this, "OptionPane.noIcon"), int1), new BasicOptionPaneUI.ButtonFactory(UIManager.getString("OptionPane.cancelButtonText", locale), this.getMnemonic("OptionPane.cancelButtonMnemonic", locale), (Icon)DefaultLookup.get(this.optionPane, this, "OptionPane.cancelIcon"), int1) };
                }
                else if (optionType == 2) {
                    array = new BasicOptionPaneUI.ButtonFactory[] { new BasicOptionPaneUI.ButtonFactory(UIManager.getString("OptionPane.okButtonText", locale), this.getMnemonic("OptionPane.okButtonMnemonic", locale), (Icon)DefaultLookup.get(this.optionPane, this, "OptionPane.okIcon"), int1), new BasicOptionPaneUI.ButtonFactory(UIManager.getString("OptionPane.cancelButtonText", locale), this.getMnemonic("OptionPane.cancelButtonMnemonic", locale), (Icon)DefaultLookup.get(this.optionPane, this, "OptionPane.cancelIcon"), int1) };
                }
                else {
                    array = new BasicOptionPaneUI.ButtonFactory[] { new BasicOptionPaneUI.ButtonFactory(UIManager.getString("OptionPane.okButtonText", locale), this.getMnemonic("OptionPane.okButtonMnemonic", locale), (Icon)DefaultLookup.get(this.optionPane, this, "OptionPane.okIcon"), int1) };
                }
                return array;
            }
            else {
                return options;
            }
        }
        else {
            return null;
        }
    }
    
    private int getMnemonic(String key, Locale l) {
        String s = (String)UIManager.get(key, l);
        if (s == null) {
            return 0;
        }
        try {
            return Integer.parseInt(s);
        }
        catch (NumberFormatException ex) {
            return 0;
        }
    }
    
    protected boolean getSizeButtonsToSameWidth() {
        return true;
    }
    
    protected int getInitialValueIndex() {
        if (this.optionPane != null) {
            Object initialValue = this.optionPane.getInitialValue();
            Object[] options = this.optionPane.getOptions();
            if (options == null) {
                return 0;
            }
            if (initialValue != null) {
                int i = options.length - 1;
                while (i >= 0) {
                    if (options[i].equals(initialValue)) {
                        return i;
                    }
                    --i;
                }
            }
        }
        return -1;
    }
    
    protected void resetInputValue() {
        if (this.inputComponent != null && this.inputComponent instanceof JTextField) {
            this.optionPane.setInputValue(((JTextField)this.inputComponent).getText());
        }
        else if (this.inputComponent != null && this.inputComponent instanceof JComboBox) {
            this.optionPane.setInputValue(((JComboBox)this.inputComponent).getSelectedItem());
        }
        else if (this.inputComponent != null) {
            this.optionPane.setInputValue(((JList)this.inputComponent).getSelectedValue());
        }
    }
    
    @Override
    public void selectInitialValue(JOptionPane optionPane) {
        if (this.inputComponent != null) {
            this.inputComponent.requestFocus();
        }
        else {
            if (this.initialFocusComponent != null) {
                this.initialFocusComponent.requestFocus();
            }
            if ((this.initialFocusComponent instanceof JButton)) {
                JRootPane rootPane = SwingUtilities.getRootPane(this.initialFocusComponent);
                if (rootPane != null) {
                    rootPane.setDefaultButton((JButton)this.initialFocusComponent);
                }
            }
        }
    }
    
    @Override
    public boolean containsCustomComponents(JOptionPane optionPane) {
        return this.hasCustomComponents;
    }
    
    private void configureMessageLabel(JLabel label) {
        Color foreground = (Color)DefaultLookup.get(this.optionPane, this, "OptionPane.messageForeground");
        if (foreground != null) {
            label.setForeground(foreground);
        }
        Font font = (Font)DefaultLookup.get(this.optionPane, this, "OptionPane.messageFont");
        if (font != null) {
            label.setFont(font);
        }
    }
    
    private void configureButton(JButton button) {
        Font font = (Font)DefaultLookup.get(this.optionPane, this, "OptionPane.buttonFont");
        if (font != null) {
            button.setFont(font);
        }
    }
    
    static BasicOptionPaneUI.Handler access$100(BasicOptionPaneUI basicOptionPaneUI) {
        return basicOptionPaneUI.getHandler();
    }
    
    static {
        BasicOptionPaneUI.newline = (String)AccessController.doPrivileged(new GetPropertyAction("line.separator"));
        if (BasicOptionPaneUI.newline == null) {
            BasicOptionPaneUI.newline = "\n";
        }
    }
    
    public static class ButtonAreaLayout implements LayoutManager
    {
        protected boolean syncAllWidths;
        protected int padding;
        protected boolean centersChildren;
        private int orientation;
        private boolean reverseButtons;
        private boolean useOrientation;
        
        public ButtonAreaLayout(boolean syncAllWidths, int padding) {
            this.syncAllWidths = syncAllWidths;
            this.padding = padding;
            this.centersChildren = true;
            this.useOrientation = false;
        }
        
        ButtonAreaLayout(boolean b, int n, int orientation, boolean reverseButtons) {
            this(b, n);
            this.useOrientation = true;
            this.orientation = orientation;
            this.reverseButtons = reverseButtons;
        }
        
        public void setSyncAllWidths(boolean syncAllWidths) {
            this.syncAllWidths = syncAllWidths;
        }
        
        public boolean getSyncAllWidths() {
            return this.syncAllWidths;
        }
        
        public void setPadding(int padding) {
            this.padding = padding;
        }
        
        public int getPadding() {
            return this.padding;
        }
        
        public void setCentersChildren(boolean centersChildren) {
            this.centersChildren = centersChildren;
            this.useOrientation = false;
        }
        
        public boolean getCentersChildren() {
            return this.centersChildren;
        }
        
        private int getOrientation(Container container) {
            if (!this.useOrientation) {
                return 0;
            }
            if (container.getComponentOrientation().isLeftToRight()) {
                return this.orientation;
            }
            switch (this.orientation) {
                case 2: {
                    return 4;
                }
                case 4: {
                    return 2;
                }
                case 0: {
                    return 0;
                }
                default: {
                    return 2;
                }
            }
        }
        
        @Override
        public void addLayoutComponent(String s, Component component) {
        }
        
        @Override
        public void layoutContainer(Container container) {
            Component[] components = container.getComponents();
            if (components != null && components.length > 0) {
                int length = components.length;
                Insets insets = container.getInsets();
                int max = 0;
                int max2 = 0;
                int n = 0;
                int n2 = 0;
                int n3 = 0;
                int n4 = container.getComponentOrientation().isLeftToRight() ? this.reverseButtons : (!this.reverseButtons);
                for (int i = 0; i < length; ++i) {
                    Dimension preferredSize = components[i].getPreferredSize();
                    max = Math.max(max, preferredSize.width);
                    max2 = Math.max(max2, preferredSize.height);
                    n += preferredSize.width;
                }
                if (this.getSyncAllWidths()) {
                    n = max * length;
                }
                int n5 = n + (length - 1) * this.padding;
                switch (this.getOrientation(container)) {
                    case 2: {
                        n2 = insets.left;
                        break;
                    }
                    case 4: {
                        n2 = container.getWidth() - insets.right - n5;
                        break;
                    }
                    case 0: {
                        if (this.getCentersChildren() || length < 2) {
                            n2 = (container.getWidth() - n5) / 2;
                            break;
                        }
                        n2 = insets.left;
                        if (this.getSyncAllWidths()) {
                            n3 = (container.getWidth() - insets.left - insets.right - n5) / (length - 1) + max;
                            break;
                        }
                        n3 = (container.getWidth() - insets.left - insets.right - n5) / (length - 1);
                        break;
                    }
                }
                for (int j = 0; j < length; ++j) {
                    j = (n4 ? (length - j - 1) : j);
                    Dimension preferredSize2 = components[j].getPreferredSize();
                    if (this.getSyncAllWidths()) {
                        components[j].setBounds(n2, insets.top, max, max2);
                    }
                    else {
                        components[j].setBounds(n2, insets.top, preferredSize2.width, preferredSize2.height);
                    }
                    if (n3 != 0) {
                        n2 += n3;
                    }
                    else {
                        n2 += components[j].getWidth() + this.padding;
                    }
                }
            }
        }
        
        @Override
        public Dimension minimumLayoutSize(Container container) {
            if (container != null) {
                Component[] components = container.getComponents();
                if (components != null && components.length > 0) {
                    int length = components.length;
                    int n = 0;
                    Insets insets = container.getInsets();
                    int n2 = insets.top + insets.bottom;
                    int n3 = insets.left + insets.right;
                    if (this.syncAllWidths) {
                        int max = 0;
                        for (int j = 0; j < length; ++j) {
                            Dimension preferredSize2 = components[j].getPreferredSize();
                            n = Math.max(n, preferredSize2.height);
                            max = Math.max(max, preferredSize2.width);
                        }
                        return new Dimension(n3 + max * length + (length - 1) * this.padding, n2 + n);
                    }
                    int n4 = 0;
                    for (int i = 0; i < length; ++i) {
                        Dimension preferredSize = components[i].getPreferredSize();
                        n = Math.max(n, preferredSize.height);
                        n4 += preferredSize.width;
                    }
                    return new Dimension(n3 + (n4 + (length - 1) * this.padding), n2 + n);
                }
            }
            return new Dimension(0, 0);
        }
        
        @Override
        public Dimension preferredLayoutSize(Container container) {
            return this.minimumLayoutSize(container);
        }
        
        @Override
        public void removeLayoutComponent(Component component) {
        }
    }
    
    public class PropertyChangeHandler implements PropertyChangeListener
    {
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            BasicOptionPaneUI.access$100(BasicOptionPaneUI.this).propertyChange(propertyChangeEvent);
        }
    }
    
    public class ButtonActionListener implements ActionListener
    {
        protected int buttonIndex;
        
        public ButtonActionListener(int buttonIndex) {
            this.buttonIndex = buttonIndex;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            if (BasicOptionPaneUI.this.optionPane != null) {
                int optionType = BasicOptionPaneUI.this.optionPane.getOptionType();
                Object[] options = BasicOptionPaneUI.this.optionPane.getOptions();
                if (BasicOptionPaneUI.this.inputComponent != null && (options != null || optionType == -1 || ((optionType == 0 || optionType == 1 || optionType == 2) && this.buttonIndex == 0))) {
                    BasicOptionPaneUI.this.resetInputValue();
                }
                if (options == null) {
                    if (optionType == 2 && this.buttonIndex == 1) {
                        BasicOptionPaneUI.this.optionPane.setValue(Integer.valueOf(2));
                    }
                    else {
                        BasicOptionPaneUI.this.optionPane.setValue(Integer.valueOf(this.buttonIndex));
                    }
                }
                else {
                    BasicOptionPaneUI.this.optionPane.setValue(options[this.buttonIndex]);
                }
            }
        }
    }
    
    private class Handler implements ActionListener, MouseListener, PropertyChangeListener
    {
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            BasicOptionPaneUI.this.optionPane.setInputValue(((JTextField)actionEvent.getSource()).getText());
        }
        
        @Override
        public void mouseClicked(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mouseReleased(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mouseEntered(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mouseExited(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mousePressed(MouseEvent mouseEvent) {
            if (mouseEvent.getClickCount() == 2) {
                JList<Object> list = (JList)mouseEvent.getSource();
                BasicOptionPaneUI.this.optionPane.setInputValue(list.getModel().getElementAt(list.locationToIndex(mouseEvent.getPoint())));
                BasicOptionPaneUI.this.optionPane.setValue(Integer.valueOf(0));
            }
        }
        
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            if (propertyChangeEvent.getSource() == BasicOptionPaneUI.this.optionPane) {
                if ("ancestor" == propertyChangeEvent.getPropertyName()) {
                    JOptionPane optionPane = (JOptionPane)propertyChangeEvent.getSource();
                    int n = propertyChangeEvent.getOldValue() == null;
                    switch (optionPane.getMessageType()) {
                        case -1: {
                            if (!(n == 0)) {
                                BasicLookAndFeel.playSound(BasicOptionPaneUI.this.optionPane, "OptionPane.informationSound");
                                break;
                            }
                            break;
                        }
                        case 3: {
                            if (!(n == 0)) {
                                BasicLookAndFeel.playSound(BasicOptionPaneUI.this.optionPane, "OptionPane.questionSound");
                                break;
                            }
                            break;
                        }
                        case 1: {
                            if (!(n == 0)) {
                                BasicLookAndFeel.playSound(BasicOptionPaneUI.this.optionPane, "OptionPane.informationSound");
                                break;
                            }
                            break;
                        }
                        case 2: {
                            if (!(n == 0)) {
                                BasicLookAndFeel.playSound(BasicOptionPaneUI.this.optionPane, "OptionPane.warningSound");
                                break;
                            }
                            break;
                        }
                        case 0: {
                            if (!(n == 0)) {
                                BasicLookAndFeel.playSound(BasicOptionPaneUI.this.optionPane, "OptionPane.errorSound");
                                break;
                            }
                            break;
                        }
                        default: {
                            System.err.println("Undefined JOptionPane type: " + optionPane.getMessageType());
                            break;
                        }
                    }
                }
                String propertyName = propertyChangeEvent.getPropertyName();
                if (propertyName == "options" || propertyName == "initialValue" || propertyName == "icon" || propertyName == "messageType" || propertyName == "optionType" || propertyName == "message" || propertyName == "selectionValues" || propertyName == "initialSelectionValue" || propertyName == "wantsInput") {
                    BasicOptionPaneUI.this.uninstallComponents();
                    BasicOptionPaneUI.this.installComponents();
                    BasicOptionPaneUI.this.optionPane.validate();
                }
                else if (propertyName == "componentOrientation") {
                    ComponentOrientation o = (ComponentOrientation)propertyChangeEvent.getNewValue();
                    JOptionPane optionPane2 = (JOptionPane)propertyChangeEvent.getSource();
                    if (o != propertyChangeEvent.getOldValue()) {
                        optionPane2.applyComponentOrientation(o);
                    }
                }
            }
        }
    }
    
    private static class MultiplexingTextField extends JTextField
    {
        private KeyStroke[] strokes;
        
        MultiplexingTextField(int columns) {
            super(columns);
        }
        
        void setKeyStrokes(KeyStroke[] strokes) {
            this.strokes = strokes;
        }
        
        @Override
        protected boolean processKeyBinding(KeyStroke keyStroke, KeyEvent e, int condition, boolean pressed) {
            int processKeyBinding = super.processKeyBinding(keyStroke, e, condition, pressed);
            if (processKeyBinding && condition != 2) {
                int i = this.strokes.length - 1;
                while (i >= 0) {
                    if (this.strokes[i].equals(keyStroke)) {
                        return false;
                    }
                    --i;
                }
            }
            return processKeyBinding;
        }
    }
    
    private static class Actions extends UIAction
    {
        private static final String CLOSE = "close";
        
        Actions(String name) {
            super(name);
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            if (this.getName() == "close") {
                ((JOptionPane)actionEvent.getSource()).setValue(Integer.valueOf(-1));
            }
        }
    }
    
    private static class ButtonFactory
    {
        private String text;
        private int mnemonic;
        private Icon icon;
        private int minimumWidth;
        
        ButtonFactory(String text, int mnemonic, Icon icon, int minimumWidth) {
            this.minimumWidth = -1;
            this.text = text;
            this.mnemonic = mnemonic;
            this.icon = icon;
            this.minimumWidth = minimumWidth;
        }
        
        JButton createButton() {
            BasicOptionPaneUI.ButtonFactory.ConstrainedButton constrainedButton;
            if (this.minimumWidth > 0) {
                constrainedButton = new BasicOptionPaneUI.ButtonFactory.ConstrainedButton(this.text, this.minimumWidth);
            }
            else {
                constrainedButton = new JButton(this.text);
            }
            if (this.icon != null) {
                constrainedButton.setIcon(this.icon);
            }
            if (this.mnemonic != 0) {
                constrainedButton.setMnemonic(this.mnemonic);
                return constrainedButton;
            }
            return constrainedButton;
        }
        
        private static class ConstrainedButton extends JButton
        {
            int minimumWidth;
            
            ConstrainedButton(String text, int minimumWidth) {
                super(text);
                this.minimumWidth = minimumWidth;
            }
            
            @Override
            public Dimension getMinimumSize() {
                Dimension minimumSize = super.getMinimumSize();
                minimumSize.width = Math.max(minimumSize.width, this.minimumWidth);
                return minimumSize;
            }
            
            @Override
            public Dimension getPreferredSize() {
                Dimension preferredSize = super.getPreferredSize();
                preferredSize.width = Math.max(preferredSize.width, this.minimumWidth);
                return preferredSize;
            }
        }
    }
}
